package com.cnbm.admin.service.impl;

import com.alibaba.fastjson.JSON;
import com.cnbm.admin.entity.LoginUser;
import com.cnbm.admin.entity.SysLogLoginEntity;
import com.cnbm.admin.enums.LoginOperationEnum;
import com.cnbm.admin.enums.LoginStatusEnum;
import com.cnbm.admin.params.LoginParam;
import com.cnbm.admin.service.CaptchaService;
import com.cnbm.admin.service.SysLogLoginService;
import com.cnbm.admin.utils.JwtUtil;
import com.cnbm.admin.service.LoginService;
import com.cnbm.common.exception.ErrorCode;
import com.cnbm.common.utils.IpUtils;
import com.cnbm.common.utils.Result;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.HttpHeaders;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * @Author weihongyang
 * @Date 2022/6/8 9:17 AM
 * @Version 1.0
 */
@Service
public class LoginServiceImpl implements LoginService {

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private AuthenticationManager authenticationManager;

    @Autowired
    private CaptchaService captchaService;

    @Autowired
    private SysLogLoginService sysLogLoginService;

    @Override
    public Result login(HttpServletRequest request, LoginParam loginParam) {
        //验证码是否正确
//        boolean flag = captchaService.validate(loginParam.getUuid(), loginParam.getCaptcha());
//        if(!flag){
//            return new Result<>().error(ErrorCode.CAPTCHA_ERROR, "验证码错误");
//        }
        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(loginParam.getUsername(),loginParam.getPassword());
        Authentication authenticate = authenticationManager.authenticate(authenticationToken);
        //登录日志
        SysLogLoginEntity log = new SysLogLoginEntity();
        log.setOperation(LoginOperationEnum.LOGIN.value());
        log.setCreateDate(new Date());
        log.setIp(IpUtils.getIpAddr(request));
        log.setUserAgent(request.getHeader(HttpHeaders.USER_AGENT));
        log.setCreatorName(loginParam.getUsername());

        if (Objects.isNull(authenticate)) {
            log.setStatus(LoginStatusEnum.FAIL.value());
            log.setCreatorName(loginParam.getUsername());
            sysLogLoginService.save(log);
            throw new RuntimeException("登录失败");
        }
        //如果认证通过了，使用userid生成一个jwt jwt存入ResponseResult返回
        LoginUser loginUser = (LoginUser) authenticate.getPrincipal();
        String userid = loginUser.getSysUserEntity().getId().toString();
        String jwt = JwtUtil.createJWT(userid);
        Map<String,String> map = new HashMap<>();
        map.put("token",jwt);
        //把完整的用户信息存入redis  userid作为key
        redisTemplate.opsForValue().set("login:"+userid,loginUser,1, TimeUnit.DAYS);

        //当前登录用户信息存入session中
        HttpSession session = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest().getSession();
        session.setAttribute(loginUser.HTTP_HEADER_NAME, JSON.toJSONString(loginUser));

        //登录成功
        log.setStatus(LoginStatusEnum.SUCCESS.value());
        log.setCreator(loginUser.getSysUserEntity().getId());
        log.setCreatorName(loginUser.getUsername());
        sysLogLoginService.save(log);
        return new Result<>().ok(map);
    }

    @Override
    public void logout(HttpServletRequest request) {
        //获取SecurityContextHolder中的用户id
        UsernamePasswordAuthenticationToken authentication = (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        Long userid = loginUser.getSysUserEntity().getId();
        //日志
		SysLogLoginEntity log = new SysLogLoginEntity();
		log.setOperation(LoginOperationEnum.LOGOUT.value());
		log.setIp(IpUtils.getIpAddr(request));
		log.setUserAgent(request.getHeader(HttpHeaders.USER_AGENT));
		log.setIp(IpUtils.getIpAddr(request));
		log.setStatus(LoginStatusEnum.SUCCESS.value());
		log.setCreator(userid);
		log.setCreatorName(loginUser.getUsername());
		log.setCreateDate(new Date());
		sysLogLoginService.save(log);
        //删除redis中的值
        redisTemplate.delete("login:"+userid);
    }

}
