package com.cnbm.admin.service.impl;

import com.cnbm.admin.dao.UserDao;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cnbm.admin.dto.SysMenuDTO;
import com.cnbm.admin.dto.SysUserDTO;
import com.cnbm.admin.entity.LoginUser;
import com.cnbm.admin.entity.SysLogLoginEntity;
import com.cnbm.admin.entity.SysMenuEntity;
import com.cnbm.admin.entity.SysUserEntity;
import com.cnbm.admin.enums.LoginOperationEnum;
import com.cnbm.admin.exception.LoginStatusException;
import com.cnbm.admin.service.SysMenuService;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.common.utils.IpUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.security.authentication.DisabledException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author weihongyang
 * @Date 2022/6/8 8:57 AM
 * @Version 1.0
 */
@Service
@Log4j2
public class UserDetailsServiceImpl implements UserDetailsService {

    @Autowired
    private UserDao userDao;

    @Autowired
    private SysMenuService sysMenuService;

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {

        LambdaQueryWrapper<SysUserEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(SysUserEntity::getUsername,username);
        SysUserEntity sysUserEntity = userDao.selectOne(lambdaQueryWrapper);

        if (Objects.isNull(sysUserEntity)) {
            throw new UsernameNotFoundException("用户名不存在");
        }
        if (sysUserEntity.getStatus() == 0) {
            throw new LoginStatusException("该账户已被禁用，请联系管理员");
        }
        log.info("sysUserEntity的值是"+sysUserEntity.toString());
        log.info("sysUserEntity.getSuperAdmin()=="+sysUserEntity.getSuperAdmin());
        Set<String> userPermissions = sysMenuService.getUserPermissions(sysUserEntity);

        List<String> list = userPermissions.stream().collect(Collectors.toList());
        log.info("list==="+list.toString());
        return new LoginUser(sysUserEntity,list);
    }
}
