package com.cnbm.generator.code.controller;

import com.cnbm.admin.annotation.LogOperation;
import com.cnbm.common.constant.Constant;
import com.cnbm.common.page.PageData;
import com.cnbm.common.utils.ExcelUtils;
import com.cnbm.common.utils.Result;
import com.cnbm.common.validator.AssertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.common.validator.group.UpdateGroup;
import com.cnbm.generator.code.dto.ChangePackagingBoxHistoryDTO;
import com.cnbm.generator.code.excel.ChangePackagingBoxHistoryExcel;
import com.cnbm.generator.code.service.ChangePackagingBoxHistoryServiceBiz;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 包装箱拆合箱历史表  前端控制器
 *
 * @author codeGenerator
 * @since  2023-02-16
 */
@RestController
@RequestMapping("/code/changePackagingBoxHistory")
@Api(tags="包装箱拆合箱历史表")
public class ChangePackagingBoxHistoryController {
    @Autowired
    private ChangePackagingBoxHistoryServiceBiz changePackagingBoxHistoryService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataTypeClass=String.class) ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataTypeClass=String.class)
    })
    @PreAuthorize("@ex.hasAuthority('code:changePackagingBoxHistory:page')")
    public Result<PageData<ChangePackagingBoxHistoryDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<ChangePackagingBoxHistoryDTO> page = changePackagingBoxHistoryService.page(params);

        return new Result<PageData<ChangePackagingBoxHistoryDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @PreAuthorize("@ex.hasAuthority('code:changePackagingBoxHistory:info')")
    public Result<ChangePackagingBoxHistoryDTO> get(@PathVariable("id") Long id){
        ChangePackagingBoxHistoryDTO data = changePackagingBoxHistoryService.get(id);

        return new Result<ChangePackagingBoxHistoryDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @PreAuthorize("@ex.hasAuthority('code:changePackagingBoxHistory:save')")
    public Result<Long> save(@RequestBody ChangePackagingBoxHistoryDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        changePackagingBoxHistoryService.save(dto);

        return new Result<Long>().ok(dto.getId());
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @PreAuthorize("@ex.hasAuthority('code:changePackagingBoxHistory:update')")
    public Result<Long> update(@RequestBody ChangePackagingBoxHistoryDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        changePackagingBoxHistoryService.update(dto);

        return new Result<Long>().ok(dto.getId());
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @PreAuthorize("@ex.hasAuthority('code:changePackagingBoxHistory:delete')")
    public Result delete(@RequestBody Long[] ids){
        //效验数据
        AssertUtils.isArrayEmpty(ids, "id");

        changePackagingBoxHistoryService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @PreAuthorize("@ex.hasAuthority('code:changePackagingBoxHistory:export')")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<ChangePackagingBoxHistoryDTO> list = changePackagingBoxHistoryService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, ChangePackagingBoxHistoryExcel.class);
    }

}