package com.cnbm.dispatchDDJ;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONObject;
import com.cnbm.dispatchDDJ.constant.QGSConstant;
import com.cnbm.dispatchDDJ.entity.DDJInfoCallBack;
import com.cnbm.dispatchDDJ.entity.TaskDTO;
import com.cnbm.dispatchDDJ.enums.ddj.ddj1.DDJPlc2WcsVar;
import com.cnbm.dispatchDDJ.enums.ddj.ddj1.DDJWcs2PlcVar;
import com.cnbm.dispatchDDJ.interfaces.Listener;
import com.cnbm.dispatchDDJ.interfaces.RuntimeEQErrListener;
import com.cnbm.dispatchDDJ.taskPool.QGSThreadPool;
import com.cnbm.s7.s7connector.enmuc.S7Client;
import com.cnbm.s7.s7connector.type.PlcVar;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collection;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.ExecutionException;
import java.util.function.BiConsumer;


/**
 * @Desc: ""
 * @Author: caixiang
 * @DATE: 2022/6/8 15:16
 */
public class DDJ1Task implements Listener {
    private static final Logger logger = LoggerFactory.getLogger(DDJ1Task.class);

    private TaskDTO taskDTO;
    private S7Client s7Client;

    public DDJ1Task(TaskDTO taskDTO, S7Client s7Client){
        this.taskDTO = taskDTO;
        this.s7Client = s7Client;
    }
    public DDJ1Task(){
    }

    public final Collection<RuntimeEQErrListener> linkTestRecvListeners = new CopyOnWriteArrayList<>();

    @Override
    public void addRuntimeEQErrListener(RuntimeEQErrListener lstnr) {
        linkTestRecvListeners.add(lstnr);
    }


    public void executeLinktestRecvTask(Integer errCode) {
        for (RuntimeEQErrListener listener : linkTestRecvListeners) {
            listener.received(errCode);
        }
    }


//    public void run()  {
//        CompletableFuture.supplyAsync(() -> {
//            System.out.println("");
//            System.out.println("开始执行 ddj1 调度任务   开始 "+ Thread.currentThread().hashCode());
//            executeLinktestRecvTask(1);
//
//            System.out.println("开始执行 ddj1 调度任务   结束"+ Thread.currentThread().hashCode());
//            return 1;
//        }, QGSThreadPool.ddj1ExecutorService).whenComplete((data, error) -> {
//            System.out.println("data:" + data+ Thread.currentThread().hashCode());
//            if (error != null) {
//                System.out.println(error);
//            }
//        });
//    }


    /* *
     *
     * status
     *      10   初始值
     *      -2   s7 数据采集失败
     *      100  任务成功完成
     *      101  任务被人为取消，取消的原因看 content
     * */
    public void run(){
        CompletableFuture.supplyAsync(() -> {
            logger.info("");
            logger.info("=================  现在开始执行 DDJ1 任务 : "+ taskDTO.toString()+"  ==================");

            //构建结果类
            DDJInfoCallBack build = new DDJInfoCallBack();
            build.setStatus(10);

            build.setTaskId(taskDTO.getMissionID());
            //调度逻辑 ------------------------------------- 开始 -------------------------------------
            //todo 这里待定，当出现脱机或者 故障的时候 怎么的一个形式告知 wms
            //步骤1          WCS监听 PlcToWcs.Operate_Mode 是否为 1 , ( 1 = 联机 )
            Integer operateMode = waitingForTarget(DDJPlc2WcsVar.Operate_Mode, 1);
            if(operateMode!=1){
                logger.info("失败"+" --- "+"步骤1.  WCS监听 DDJPlc2WcsVar.Operate_Mode 是否 等于 1 失败");
                build.setContent("失败，步骤1，监听数据异常");
                build.setStatus(-2);
                return build;
            }
            logger.info("成功"+" --- "+"步骤1.  WCS监听 DDJPlc2WcsVar.Operate_Mode 等于 1 ");


            //步骤2          WCS监听 PlcToWcs.Mission_State 是否为 0 , ( 0 = 空闲 )
            Integer missionState = waitingForTarget(DDJPlc2WcsVar.Mission_State, 0);
            if(missionState!=1){
                logger.info("失败"+" --- "+"步骤2.  WCS监听 PlcToWcs.Mission_State 是否 等于 0 失败");
                build.setContent("失败，步骤2，监听数据异常");
                build.setStatus(-2);
                return build;
            }
            logger.info("成功"+" --- "+"步骤2.  WCS监听 PlcToWcs.Mission_State 等于 0 ");

            //步骤3          WCS下发任务
            issueTask();
            logger.info("成功"+" --- "+"步骤3.  WCS 下发任务 ");


            //步骤4          WCS启动任务（WcsToPlc.Command_No 置为 1 ）
            write(s7Client,DDJWcs2PlcVar.Command_No,1);
            logger.info("成功"+" --- "+"步骤4.  WCS启动任务（WcsToPlc.Command_No 置为 1 ）成功 ");


            //步骤5          WCS 监控PlcToWcs.Mission_State ==2 监控任务是否完成
            Integer missionState2 = waitingForTargetAndLoggerError(DDJPlc2WcsVar.Mission_State,DDJPlc2WcsVar.Operate_Mode, 2);
            if(missionState2!=1){
                logger.info("失败"+" --- "+"步骤5.  WCS 监控PlcToWcs.Mission_State ==2 监控任务是否完成");
                build.setContent("失败，步骤5，监听数据异常");
                build.setStatus(-2);
                return build;
            }
            logger.info("成功"+" --- "+"步骤5. WCS 监控PlcToWcs.Mission_State ==2  ");

            //步骤6          WCS 把 Command_No 置为 2（重置操作）
            write(s7Client,DDJWcs2PlcVar.Command_No,2);
            logger.info("成功"+" --- "+"步骤6.  WCS 把 Command_No 置为 2（重置操作） ");
            build.setStatus(100);
            build.setContent("任务成功完成");
            //调度逻辑 ------------------------------------- 结束


            return build;
        }, QGSThreadPool.ddj1ExecutorService).whenComplete(new BiConsumer<DDJInfoCallBack, Throwable>() {
            //任务完成以后把结果返回给 第调用者
            @Override
            public void accept(DDJInfoCallBack agvInfoCallBack, Throwable throwable) {
                if(throwable!=null){
                    logger.info("whenComplete throwable is " + throwable.getMessage());
                }else {
                    logger.info("任务执行完成，现在wcs把 结果："+agvInfoCallBack.toString()+", 推送给wms");
                    HttpUtil.post(QGSConstant.wmsUrl, JSONObject.toJSONString(agvInfoCallBack));
                    logger.info("=================  任务结束 : "+ taskDTO.toString()+"  ==================");
                    logger.info("");

                }
            }
        });

    }




    /**
     * desc: 下发任务给电控
     * return：
     *      true  成功
     *      false 失败
     * */
    private void issueTask() {
        write(s7Client,DDJWcs2PlcVar.Mission_Type, taskDTO.getMissionType());

        write(s7Client,DDJWcs2PlcVar.From_Row, taskDTO.getFromRow());
        write(s7Client,DDJWcs2PlcVar.From_Layer, taskDTO.getFromLayer());
        write(s7Client,DDJWcs2PlcVar.From_List, taskDTO.getFromList());
        write(s7Client,DDJWcs2PlcVar.To_Row, taskDTO.getToRow());
        write(s7Client,DDJWcs2PlcVar.To_List, taskDTO.getToList());
        write(s7Client,DDJWcs2PlcVar.To_Layer, taskDTO.getToLayer());

        write(s7Client,DDJWcs2PlcVar.Goods_Size, taskDTO.getGoodsSiZe());
        write(s7Client,DDJWcs2PlcVar.Coordinate_conversion, taskDTO.getCoordinateConversion());
        write(s7Client,DDJWcs2PlcVar.Mission_ID, taskDTO.getMissionID());
        write(s7Client,DDJWcs2PlcVar.Pallet_Num, taskDTO.getPalletNum());
    }

    /**
     *
     * arg:
     *      RGV1Plc2WcsVarEnum 参数类型
     *      targetValue        目的值
     * return:
     *      2 opc 访问异常;
     *      1 成功;
     * */
    private Integer waitingForTarget(DDJPlc2WcsVar var, Integer targetValue) {
        Integer res = 0;
        while (true){
            try {
                Integer now = (Integer) read(s7Client,var);
                if(targetValue.equals(now)){
                    res = 1;
                    break;
                }
                Thread.sleep(200);
            }catch (Exception e){
                logger.info("S7 数据采集 异常。");
                res = 2;
                break;
            }
        }
        return res;
    }

    private Integer waitingForTargetAndOperaMode(DDJPlc2WcsVar var, Integer targetValue) {
        Integer res = 0;
        while (true){
            try {
                Integer now = (Integer) read(s7Client,var);
                if(targetValue.equals(now)){
                    res = 1;
                    break;
                }
                Thread.sleep(200);
            }catch (Exception e){
                logger.info("S7 数据采集 异常。");
                res = 2;
                break;
            }
        }
        return res;
    }

    private Integer waitingForTargetAndLoggerError(DDJPlc2WcsVar var,DDJPlc2WcsVar var2, Integer targetValue) {
        Integer res = 0;
        while (true){
            try {
                Integer now = (Integer) read(s7Client,var);
                Integer operateMode = (Integer) read(s7Client,var2);
                if(targetValue.equals(now)){
                    res = 1;
                    break;
                }
                if(operateMode.equals(2)){
                    Integer faultNo = (Integer) read(s7Client,DDJPlc2WcsVar.Fault_No);
                    logger.info("步骤5  DDJ在运行的时候出现 fault, fault no = "+faultNo);
                    //调用上层的 回调方法     开始
                    executeLinktestRecvTask(faultNo);
                    //调用上层的 回调方法     结束
                    Thread.sleep(10000);
                }
                Thread.sleep(200);
            }catch (Exception e){
                logger.info("S7 数据采集 异常。");
                res = 2;
                break;
            }
        }
        return res;
    }


    /**
     * return
     *          成功： 返回相应的object对象
     *          失败： 返回null
     * */
    private Object read(S7Client s7Client, DDJPlc2WcsVar var) {
        try {
            return s7Client.read(var.getArea(), var.getAreaNumber(), var.getByteOffset(), var.getBitOffset(), var.getLength(), var.getStrSize(), var.getType());
        }catch (Exception e){
            logger.error("host:"+s7Client.getHost()+" ; read 操作出现问题: "+e.getMessage());
            e.printStackTrace();
            return null;
        }
    }
    private void write(S7Client s7Client, DDJWcs2PlcVar var, Object newValue) {
        if(var.getType().equals(PlcVar.STRING_Array)){
            String[] s = (String[])newValue;
            String[] ss = (String[])newValue;
            if(s.length > var.getLength() ){
                ss = new String[var.getLength()];
                for(int i=0;i< var.getLength();i++){
                    ss[i] = s[i];
                }
            }
            s7Client.write(var.getArea(), var.getAreaNumber(), var.getByteOffset(), var.getBitOffset(), var.getStrSize(), var.getType(),ss);
        }else {
            s7Client.write(var.getArea(), var.getAreaNumber(), var.getByteOffset(), var.getBitOffset(), var.getStrSize(), var.getType(),newValue);
        }
    }


}
