package com.cnbm.packing.controller;

import com.cnbm.admin.annotation.LogOperation;
import com.cnbm.common.constant.Constant;
import com.cnbm.common.page.PageData;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.common.utils.ExcelUtils;
import com.cnbm.common.utils.Result;
import com.cnbm.common.validator.AssertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.common.validator.group.UpdateGroup;
import com.cnbm.packing.dto.ChangePackagingBoxHistoryDTO;
import com.cnbm.packing.entity.ChangePackagingBoxHistory;
import com.cnbm.packing.excel.ChangePackagingBoxHistoryExcel;
import com.cnbm.packing.service.ChangePackagingBoxHistoryServiceBiz;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Map;


/**
 * 包装箱拆合箱历史表  前端控制器
 *
 * @author packingGenerator
 * @since  2023-02-16
 */
@RestController
@RequestMapping("/packing/changePackagingBoxHistory")
@Api(tags="包装箱拆合箱历史表")
public class ChangePackagingBoxHistoryController {
    @Autowired
    private ChangePackagingBoxHistoryServiceBiz changePackagingBoxHistoryService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataTypeClass=String.class) ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataTypeClass=String.class),
        @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataTypeClass = LocalDateTime.class),
        @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataTypeClass = LocalDateTime.class),
        @ApiImplicitParam(name = "type", value = "类型，1=加入包装箱；2=移除包装箱；3=换箱", paramType = "query", dataTypeClass=Integer.class)
    })
    @PreAuthorize("@ex.hasAuthority('packing:changePackagingBoxHistory:page')")
    public Result<PageData<ChangePackagingBoxHistoryDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<ChangePackagingBoxHistoryDTO> page = changePackagingBoxHistoryService.page(params);

        return new Result<PageData<ChangePackagingBoxHistoryDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @PreAuthorize("@ex.hasAuthority('packing:changePackagingBoxHistory:info')")
    public Result<ChangePackagingBoxHistoryDTO> get(@PathVariable("id") Long id){
        ChangePackagingBoxHistoryDTO data = changePackagingBoxHistoryService.get(id);

        return new Result<ChangePackagingBoxHistoryDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @PreAuthorize("@ex.hasAuthority('packing:changePackagingBoxHistory:save')")
    public Result<Long> save(@RequestBody ChangePackagingBoxHistoryDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        changePackagingBoxHistoryService.save(dto);

        return new Result<Long>().ok(dto.getId());
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @PreAuthorize("@ex.hasAuthority('packing:changePackagingBoxHistory:update')")
    public Result<Long> update(@RequestBody ChangePackagingBoxHistoryDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        changePackagingBoxHistoryService.update(dto);

        return new Result<Long>().ok(dto.getId());
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @PreAuthorize("@ex.hasAuthority('packing:changePackagingBoxHistory:delete')")
    public Result delete(@RequestBody Long[] ids){
        //效验数据
        AssertUtils.isArrayEmpty(ids, "id");

        changePackagingBoxHistoryService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataTypeClass = LocalDateTime.class),
            @ApiImplicitParam(name = "type", value = "类型，1=加入包装箱；2=移除包装箱；3=换箱", paramType = "query", dataTypeClass=Integer.class)
    })
    @PreAuthorize("@ex.hasAuthority('packing:changePackagingBoxHistory:export')")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<ChangePackagingBoxHistory> list = changePackagingBoxHistoryService.historyList(params);
        List<ChangePackagingBoxHistoryDTO> dtoList = ConvertUtils.sourceToTarget(list, ChangePackagingBoxHistoryDTO.class);
        if(list.size()>0) {
            for(ChangePackagingBoxHistoryDTO dto:dtoList){
                //操作时间 转化
                DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                if(dto.getCreateTime()!=null){
                    dto.setCreateTime1(df.format(dto.getCreateTime()));
                }
                //操作类型 1=模组加入包装箱；2=模组移出包装箱；3=模组换箱
                if(dto.getType()==1){
                    dto.setType1("模组加入包装箱");
                }
                if(dto.getType()==2){
                    dto.setType1("模组移出包装箱");
                }
                if(dto.getType()==3){
                    dto.setType1("模组换箱");
                }
            }
            ExcelUtils.exportExcelToTarget(response, "包装箱模组操作记录", dtoList, ChangePackagingBoxHistoryExcel.class);
        }
    }

}