package com.cnbm.packing.controller;

import com.cnbm.admin.annotation.LogOperation;
import com.cnbm.common.constant.Constant;
import com.cnbm.common.page.PageData;
import com.cnbm.common.utils.ExcelUtils;
import com.cnbm.common.utils.Result;
import com.cnbm.common.validator.AssertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.common.validator.group.UpdateGroup;

import com.cnbm.packing.dto.WoPackagingPrintHistoryDTO;
import com.cnbm.packing.excel.WoPackagingPrintHistoryExcel;
import com.cnbm.packing.service.WoPackagingPrintHistoryServiceBiz;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;


/**
 * 包装箱打印历史表  前端控制器
 *
 * @author packingGenerator
 * @since  2023-02-15
 */
@RestController
@RequestMapping("/packing/woPackagingPrintHistory")
@Api(tags="包装箱打印历史表")
public class WoPackagingPrintHistoryController {
    @Autowired
    private WoPackagingPrintHistoryServiceBiz woPackagingPrintHistoryService;


    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataTypeClass=String.class) ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataTypeClass=String.class),
        @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataTypeClass = LocalDateTime.class),
        @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataTypeClass = LocalDateTime.class),
        @ApiImplicitParam(name = "boxNo", value = "包装箱编号", paramType = "query", dataTypeClass = String.class)
    })
    @PreAuthorize("@ex.hasAuthority('packing:woPackagingPrintHistory:page')")
    public Result<PageData<WoPackagingPrintHistoryDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<WoPackagingPrintHistoryDTO> page = woPackagingPrintHistoryService.page(params);

        return new Result<PageData<WoPackagingPrintHistoryDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @PreAuthorize("@ex.hasAuthority('packing:woPackagingPrintHistory:info')")
    public Result<WoPackagingPrintHistoryDTO> get(@PathVariable("id") Long id){
        WoPackagingPrintHistoryDTO data = woPackagingPrintHistoryService.get(id);

        return new Result<WoPackagingPrintHistoryDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @PreAuthorize("@ex.hasAuthority('packing:woPackagingPrintHistory:save')")
    public Result<Long> save(@RequestBody WoPackagingPrintHistoryDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        woPackagingPrintHistoryService.save(dto);

        return new Result<Long>().ok(dto.getId());
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @PreAuthorize("@ex.hasAuthority('packing:woPackagingPrintHistory:update')")
    public Result<Long> update(@RequestBody WoPackagingPrintHistoryDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        woPackagingPrintHistoryService.update(dto);

        return new Result<Long>().ok(dto.getId());
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @PreAuthorize("@ex.hasAuthority('packing:woPackagingPrintHistory:delete')")
    public Result delete(@RequestBody Long[] ids){
        //效验数据
        AssertUtils.isArrayEmpty(ids, "id");

        woPackagingPrintHistoryService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @PreAuthorize("@ex.hasAuthority('packing:woPackagingPrintHistory:export')")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<WoPackagingPrintHistoryDTO> list = woPackagingPrintHistoryService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, WoPackagingPrintHistoryExcel.class);
    }

    @PostMapping("print/{id}")
    @ApiOperation("打印记录新增")
    @LogOperation("打印记录新增")
    public Result<Long> print(@PathVariable("id") Long id) {

        woPackagingPrintHistoryService.print(id);

        return new Result();
    }

}