package com.cnbm.packing.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cnbm.admin.utils.BaseSupportUtils;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.packing.dto.ChangePackingBoxDTO;
import com.cnbm.packing.dto.WoPackagingBoxSubstrateDTO;
import com.cnbm.packing.entity.ChangePackagingBoxHistory;
import com.cnbm.packing.entity.WoPackagingBoxSubstrate;
import com.cnbm.packing.mapper.WoPackagingBoxSubstrateMapper;
import com.cnbm.packing.param.PackingInfoQueryParam;
import com.cnbm.packing.service.ChangePackagingBoxHistoryServiceBiz;
import com.cnbm.packing.service.WoPackagingBoxSubstrateServiceBiz;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 包装箱基板关联表
 *
 * @author codeGenerator
 * @since 2023-02-16
 */
@Service
public class WoPackagingBoxSubstrateServiceBizImpl extends CrudServiceImpl<WoPackagingBoxSubstrateMapper, WoPackagingBoxSubstrate, WoPackagingBoxSubstrateDTO> implements WoPackagingBoxSubstrateServiceBiz {

    @Autowired
    private ChangePackagingBoxHistoryServiceBiz changePackagingBoxHistoryService;
    @Autowired
    private WoPackagingBoxSubstrateMapper mapper;

    @Override
    public QueryWrapper<WoPackagingBoxSubstrate> getWrapper(Map<String, Object> params){
        String packagingBoxId = (String) params.get("packagingBoxId");
        String woSubstrateId = (String)params.get("woSubstrateId");

        QueryWrapper<WoPackagingBoxSubstrate> wrapper = new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(packagingBoxId), WoPackagingBoxSubstrate.PACKAGING_BOX_ID, packagingBoxId);
        wrapper.like(StringUtils.isNotBlank(woSubstrateId), WoPackagingBoxSubstrate.WO_SUBSTRATE_ID, woSubstrateId);

        return wrapper;
    }



    @Override
    public PageData<WoPackagingBoxSubstrateDTO> page (Map<String, Object> params){
        IPage<WoPackagingBoxSubstrate> page = baseDao.selectPage(
                getPage(params, WoPackagingBoxSubstrate.CREATE_TIME, false),
                getWrapper(params)
        );
        return getPageData(page, WoPackagingBoxSubstrateDTO.class);
    }

    @Override
    public WoPackagingBoxSubstrateDTO get(Long id) {
        WoPackagingBoxSubstrate entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, WoPackagingBoxSubstrateDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(WoPackagingBoxSubstrateDTO dto) {
        WoPackagingBoxSubstrate entity = ConvertUtils.sourceToTarget(dto, WoPackagingBoxSubstrate.class);
        BaseSupportUtils.setCommonField(entity);
        insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(WoPackagingBoxSubstrateDTO dto) {
        WoPackagingBoxSubstrate entity = ConvertUtils.sourceToTarget(dto, WoPackagingBoxSubstrate.class);
        BaseSupportUtils.setUpdateCommonField(entity);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void removeSubstrate(Long id) {
        WoPackagingBoxSubstrate entity = baseDao.selectById(id);
        //移箱写入包装箱拆合箱历史表
        ChangePackagingBoxHistory changePackagingBoxHistory = new ChangePackagingBoxHistory();
        changePackagingBoxHistory.setSubstrateId(entity.getWoSubstrateId());
        changePackagingBoxHistory.setSourceBoxNo(entity.getPackagingBoxId());
        changePackagingBoxHistory.setSourceSlot(entity.getSlot());
        changePackagingBoxHistory.setLeaveTime(LocalDateTime.now());
        changePackagingBoxHistory.setType(2);
        BaseSupportUtils.setCommonField(changePackagingBoxHistory);
        changePackagingBoxHistoryService.insert(changePackagingBoxHistory);
        //模组从该包装箱中移出，该模组变为未绑定BoxID的模组
        UpdateWrapper<WoPackagingBoxSubstrate> wrapper = new UpdateWrapper<>();
        wrapper.set(WoPackagingBoxSubstrate.PACKAGING_BOX_ID,null);
        wrapper.set(WoPackagingBoxSubstrate.SLOT,null);
        wrapper.eq(WoPackagingBoxSubstrate.ID,entity.getId());
        update(entity,wrapper);
//        entity.setPackagingBoxId("");
//        entity.setSlot(null);
//        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void insertSubstrate(ChangePackingBoxDTO dto) {
        WoPackagingBoxSubstrate entity = baseDao.selectById(dto.getId());
        //合箱写入包装箱拆合箱历史表
        ChangePackagingBoxHistory changePackagingBoxHistory = new ChangePackagingBoxHistory();
        changePackagingBoxHistory.setSubstrateId(dto.getWoSubstrateId());
        changePackagingBoxHistory.setTargetBoxNo(dto.getPackagingBoxId());
        changePackagingBoxHistory.setTargetSlot(dto.getSlot());
        changePackagingBoxHistory.setInputTime(LocalDateTime.now());
        changePackagingBoxHistory.setType(1);
        BaseSupportUtils.setCommonField(changePackagingBoxHistory);
        changePackagingBoxHistoryService.insert(changePackagingBoxHistory);
        //更新
        entity.setPackagingBoxId(dto.getPackagingBoxId());
        entity.setSlot(dto.getSlot());
        BaseSupportUtils.setUpdateCommonField(entity);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void replaceSubstrate(ChangePackingBoxDTO[] dtos) {
        for(ChangePackingBoxDTO dto : dtos){
            WoPackagingBoxSubstrate entity = baseDao.selectById(dto.getId());
            //换箱写入包装箱拆合箱历史表
            ChangePackagingBoxHistory changePackagingBoxHistory = new ChangePackagingBoxHistory();
            changePackagingBoxHistory.setSubstrateId(entity.getWoSubstrateId());
            changePackagingBoxHistory.setSourceBoxNo(entity.getPackagingBoxId());
            changePackagingBoxHistory.setSourceSlot(entity.getSlot());
            changePackagingBoxHistory.setTargetBoxNo(dto.getPackagingBoxId());
            changePackagingBoxHistory.setTargetSlot(dto.getSlot());
            changePackagingBoxHistory.setType(3);
            BaseSupportUtils.setCommonField(changePackagingBoxHistory);
            changePackagingBoxHistoryService.insert(changePackagingBoxHistory);
            //更新
            entity.setPackagingBoxId(dto.getPackagingBoxId());
            entity.setSlot(dto.getSlot());
            BaseSupportUtils.setUpdateCommonField(entity);
            updateById(entity);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void insertSubstrateManual(ChangePackingBoxDTO[] dtos) {
        for(ChangePackingBoxDTO dto : dtos) {
//            QueryWrapper<WoPackagingBoxSubstrate> wrapper = new QueryWrapper<>();
//            wrapper.eq(StringUtils.isNotBlank(dto.getWoSubstrateId()), WoPackagingBoxSubstrate.WO_SUBSTRATE_ID, dto.getWoSubstrateId());
            WoPackagingBoxSubstrate entity = ConvertUtils.sourceToTarget(dto, WoPackagingBoxSubstrate.class);
            BaseSupportUtils.setCommonField(entity);
            insert(entity);
            /*
            if (mapper.selectCount(wrapper) > 0 && StringUtils.isNotBlank(dto.getWoSubstrateId())) {
                WoPackagingBoxSubstrate substrate = mapper.selectList(wrapper).get(0);
                substrate.setPackagingBoxId(dto.getPackagingBoxId());
                BaseSupportUtils.setUpdateCommonField(substrate);
                updateById(substrate);
            } else {
                //模组ID有时为空，用户会输入”无码“
                if (dto.getWoSubstrateId() == null) {
                    dto.setWoSubstrateId("无码");
                }
                //效验数据
                ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);
                WoPackagingBoxSubstrate entity = ConvertUtils.sourceToTarget(dto, WoPackagingBoxSubstrate.class);
                BaseSupportUtils.setCommonField(entity);
                insert(entity);
            }
             */
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateSubstrateManual(ChangePackingBoxDTO[] dtos) {
        for(ChangePackingBoxDTO dto : dtos) {
            //判断模组id是否非空
            if(StringUtils.isBlank(dto.getWoSubstrateId())) {
                deleteById(dto.getId());
            }
            else {
                WoPackagingBoxSubstrate entity = ConvertUtils.sourceToTarget(dto, WoPackagingBoxSubstrate.class);
                BaseSupportUtils.setUpdateCommonField(entity);
                updateById(entity);
            }
        }
    }

    @Override
    public List<WoPackagingBoxSubstrateDTO> substrateList(Map<String, Object> params) {
        return mapper.substrateList(params);
    }

    @Override
    public PageData<WoPackagingBoxSubstrateDTO> substratePage(PackingInfoQueryParam param) {
        IPage<WoPackagingBoxSubstrateDTO> page = mapper.substratePage(
                new Page<>(param.getCurrent(), param.getSize()),
                param);
        return getPageData(page, WoPackagingBoxSubstrateDTO.class);
    }

    @Override
    public WoPackagingBoxSubstrate getBySubId(String subId) {
        return mapper.getOneByWoSubstrateId(subId);
    }

    @Override
    public int updatePackagingBoxIdAndSlotByWoSubstrateId(String packagingBoxId, Integer slot , String woSubstrateId) {
        return mapper.updatePackagingBoxIdAndSlotByWoSubstrateId(packagingBoxId,slot,woSubstrateId);
    }
}