package com.cnbm.packing.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.cnbm.admin.utils.BaseSupportUtils;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.packing.dto.WoPackagingBoxDTO;
import com.cnbm.packing.dto.WoPackagingPrintHistoryDTO;
import com.cnbm.packing.entity.WoPackagingPrintHistory;
import com.cnbm.packing.mapper.WoPackagingPrintHistoryMapper;
import com.cnbm.packing.service.WoPackagingBoxServiceBiz;
import com.cnbm.packing.service.WoPackagingPrintHistoryServiceBiz;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 包装箱打印历史表
 *
 * @author codeGenerator
 * @since 2023-02-15
 */
@Service
public class WoPackagingPrintHistoryServiceBizImpl extends CrudServiceImpl<WoPackagingPrintHistoryMapper, WoPackagingPrintHistory, WoPackagingPrintHistoryDTO> implements WoPackagingPrintHistoryServiceBiz {

    @Autowired
    private WoPackagingPrintHistoryMapper mapper;

    @Autowired
    private WoPackagingBoxServiceBiz woPackagingBoxServiceBiz;

    @Override
    public QueryWrapper<WoPackagingPrintHistory> getWrapper(Map<String, Object> params){

        QueryWrapper<WoPackagingPrintHistory> wrapper = new QueryWrapper<>();

        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        if(params.get("startTime")!=null && params.get("endTime")!=null){
            LocalDateTime startTime = LocalDateTime.parse(params.get("startTime").toString(),df);
            LocalDateTime endTime = LocalDateTime.parse(params.get("endTime").toString(),df);
            wrapper.between(startTime!=null && endTime!=null, WoPackagingPrintHistory.PRINT_TIME, startTime, endTime);
        }
        String boxNo = (String) params.get("boxNo");
        wrapper.like(StringUtils.isNotBlank(boxNo), WoPackagingPrintHistory.BOX_NO, boxNo);

        return wrapper;
    }

    @Override
    public PageData<WoPackagingPrintHistoryDTO> page (Map<String, Object> params){
        IPage<WoPackagingPrintHistory> page = baseDao.selectPage(
                getPage(params, WoPackagingPrintHistory.PRINT_TIME, false),
                getWrapper(params)
        );
        return getPageData(page, WoPackagingPrintHistoryDTO.class);
    }

    @Override
    public WoPackagingPrintHistoryDTO get(Long id) {
        WoPackagingPrintHistory entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, WoPackagingPrintHistoryDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(WoPackagingPrintHistoryDTO dto) {
        WoPackagingPrintHistory entity = ConvertUtils.sourceToTarget(dto, WoPackagingPrintHistory.class);
        BaseSupportUtils.setCommonField(entity);
        insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(WoPackagingPrintHistoryDTO dto) {
        WoPackagingPrintHistory entity = ConvertUtils.sourceToTarget(dto, WoPackagingPrintHistory.class);
        BaseSupportUtils.setUpdateCommonField(entity);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<WoPackagingPrintHistoryDTO> list() {
        List<WoPackagingPrintHistoryDTO> list = mapper.list();
        return list;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void print(Long id) {

        WoPackagingBoxDTO woPackagingBox = woPackagingBoxServiceBiz.get(id);
        //新增包装箱打印记录
        WoPackagingPrintHistory woPackagingPrintHistory = new WoPackagingPrintHistory();
//        BeanUtils.copyProperties(woPackagingBox, woPackagingPrintHistory);
//        woPackagingPrintHistory.setId(null);
        woPackagingPrintHistory.setBoxNo(woPackagingBox.getBoxNo());
        woPackagingPrintHistory.setPrintStatus(1);
        woPackagingPrintHistory.setPrintCount(1);
        LocalDateTime printTime = LocalDateTime.now();
        woPackagingPrintHistory.setPrintTime(printTime);
        BaseSupportUtils.setCommonField(woPackagingPrintHistory);
        insert(woPackagingPrintHistory);

        //更新包装箱表中打印状态和时间
        woPackagingBox.setPrintTime(printTime);
        if(woPackagingBox.getPrintStatus()==0){
            woPackagingBox.setPrintCount(1);
            woPackagingBox.setPrintStatus(1);
        }
        else{
            woPackagingBox.setPrintCount(woPackagingBox.getPrintCount()+1);
        }
        BaseSupportUtils.setUpdateCommonField(woPackagingBox);
        woPackagingBoxServiceBiz.update(woPackagingBox);
    }


}