package com.cnbm.s7.retrydemo;

import com.cnbm.s7.s7connector.exception.S7CheckResultException;
import org.springframework.remoting.RemoteAccessException;
import org.springframework.retry.backoff.FixedBackOffPolicy;
import org.springframework.retry.policy.SimpleRetryPolicy;
import org.springframework.retry.support.RetryTemplate;

import java.util.HashMap;
import java.util.Map;

public class RetryMain {
    public static void main(String[] args) {

        /**
         * 重试间隔时间ms,默认1000ms
         * */
        long fixedPeriodTime = 1000L;
        /**
         * 最大重试次数,默认为3
         */
        int maxRetryTimes = 3;
        /**
         * 表示哪些异常需要重试,key表示异常的字节码,value为true表示需要重试
         */
        Map<Class<? extends Throwable>, Boolean> exceptionMap = new HashMap<>();


        exceptionMap.put(RemoteAccessException.class,true);
        exceptionMap.put(S7CheckResultException.class,true);

        // 构建重试模板实例
        RetryTemplate retryTemplate = new RetryTemplate();

        // 设置重试回退操作策略，主要设置重试间隔时间
        FixedBackOffPolicy backOffPolicy = new FixedBackOffPolicy();
        backOffPolicy.setBackOffPeriod(fixedPeriodTime);

        // 设置重试策略，主要设置重试次数
        SimpleRetryPolicy retryPolicy = new SimpleRetryPolicy(maxRetryTimes, exceptionMap);

        retryTemplate.setRetryPolicy(retryPolicy);
        retryTemplate.setBackOffPolicy(backOffPolicy);

        Boolean execute = retryTemplate.execute(
                //RetryCallback
                retryContext -> {
                    boolean b = RetryDemoTask.retryTask2("abc");
                    System.err.println("retry-main : 调用的结果:"+b+",times:"+retryContext.getRetryCount()+"  hashCode:"+Thread.currentThread().hashCode());
                    return b;
                },
                retryContext -> {
                    //RecoveryCallback
                    System.err.println("retry-main : 已达到最大重试次数或抛出了不重试的异常~~~"+"  hashCode:"+Thread.currentThread().hashCode());
                    return false;
                }
        );
        System.err.println("retry-main : 执行结果:"+execute);
    }
}
